@@ -508,6 +508,7 @@ finish_exit_async (GObject *source_object,
                                                   &error);
     g_assert (result);
     g_debug ("ibus_bus_exit_finish: OK");
+    g_usleep (G_USEC_PER_SEC);
     call_next_async_function ();
 }
 
@@ -529,6 +530,42 @@ test_async_apis_finish (gpointer user_data)
     return FALSE;
 }
 
+static void
+test_get_engines_by_names (void)
+{
+    IBusEngineDesc **engines = NULL;
+    const gchar *names[] = {
+        "xkb:us::eng",
+        "xkb:ca:eng:eng",
+        "xkb:fr::fra",
+        "xkb:jp::jpn",
+        "invalid_engine_name",
+        NULL,
+    };
+
+    engines = ibus_bus_get_engines_by_names (bus, names);
+
+    g_assert(engines != NULL);
+    IBusEngineDesc **p;
+
+    gint i = 0;
+    for (p = engines; *p != NULL; p++) {
+        g_assert (IBUS_IS_ENGINE_DESC (*p));
+        g_assert_cmpstr (names[i], ==, ibus_engine_desc_get_name (*p));
+        i++;
+        g_object_unref (*p);
+        // The ref should be zero, *p is released.
+        g_assert (!IBUS_IS_ENGINE_DESC (*p));
+    }
+
+    // The last engine does not exist.
+    g_assert_cmpint (i, ==, G_N_ELEMENTS(names) - 2);
+
+    g_free (engines);
+
+    engines = NULL;
+}
+
 static void
 test_async_apis (void)
 {
@@ -567,6 +604,23 @@ call_next_async_function (void)
         (*async_functions[index++])();
 }
 
+static void
+_bus_connected_cb (IBusBus *bus,
+                   gpointer user_data)
+{
+    g_assert (ibus_bus_is_connected (bus));
+    ibus_quit ();
+}
+
+static void
+test_bus_new_async (void)
+{
+    g_object_unref (bus);
+    bus = ibus_bus_new_async ();
+    g_signal_connect (bus, "connected", G_CALLBACK (_bus_connected_cb), NULL);
+    ibus_main ();
+}
+
 gint
 main (gint    argc,
       gchar **argv)
@@ -576,12 +630,22 @@ main (gint    argc,
     g_test_init (&argc, &argv, NULL);
     ibus_init ();
     bus = ibus_bus_new ();
+    g_object_unref (bus);
+    bus = ibus_bus_new (); // crosbug.com/17293
 
     g_test_add_func ("/ibus/list-engines", test_list_engines);
     g_test_add_func ("/ibus/list-active-engines", test_list_active_engines);
     g_test_add_func ("/ibus/create-input-context-async",
                      test_create_input_context_async);
+    g_test_add_func ("/ibus/get-engines-by-names", test_get_engines_by_names);
     g_test_add_func ("/ibus/async-apis", test_async_apis);
+    g_test_add_func ("/ibus/bus-new-async", test_bus_new_async);
+    g_test_add_func ("/ibus/bus-new-async/list-engines", test_list_engines);
+    g_test_add_func ("/ibus/bus-new-async/list-active-engines", test_list_active_engines);
+    g_test_add_func ("/ibus/bus-new-async/create-input-context-async",
+                     test_create_input_context_async);
+    g_test_add_func ("/ibus/bus-new-async/get-engines-by-names", test_get_engines_by_names);
+    g_test_add_func ("/ibus/bus-new-async/async-apis", test_async_apis);
 
     result = g_test_run ();
     g_object_unref (bus);
