@@ -3,8 +3,8 @@
 static void
 test_keyname (void)
 {
-    g_assert_cmpstr (ibus_keyval_name (IBUS_Home), ==, "Home");
-    g_assert (ibus_keyval_from_name ("Home") == IBUS_Home);
+    g_assert_cmpstr (ibus_keyval_name (IBUS_KEY_Home), ==, "Home");
+    g_assert (ibus_keyval_from_name ("Home") == IBUS_KEY_Home);
 }
 
 gint
